# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.interface import Interface, InterfaceMeta
from .keyboard import Keyboard
from .pointer import Pointer
from .touch import Touch

import enum
import six


@six.add_metaclass(InterfaceMeta)
class Seat(Interface):
    """group of input devices

    A seat is a group of keyboards, pointer and touch devices. This
    object is published as a global during start up, or when such a
    device is hot plugged.  A seat typically has a pointer and
    maintains a keyboard focus and a pointer focus.
    """
    name = "wl_seat"
    version = 6

    capability = enum.Enum("capability", {
        "pointer": 1,
        "keyboard": 2,
        "touch": 4,
    })


@Seat.request("n", [Pointer])
def get_pointer(self):
    """return pointer object

    The ID provided will be initialized to the wl_pointer interface
    for this seat.

    This request only takes effect if the seat has the pointer
    capability, or has had the pointer capability in the past.
    It is a protocol violation to issue this request on a seat that has
    never had the pointer capability.

    :returns: :class:`Pointer` -- seat pointer
    """
    id = self._marshal_constructor(0, Pointer)
    return id


@Seat.request("n", [Keyboard])
def get_keyboard(self):
    """return keyboard object

    The ID provided will be initialized to the wl_keyboard interface
    for this seat.

    This request only takes effect if the seat has the keyboard
    capability, or has had the keyboard capability in the past.
    It is a protocol violation to issue this request on a seat that has
    never had the keyboard capability.

    :returns: :class:`Keyboard` -- seat keyboard
    """
    id = self._marshal_constructor(1, Keyboard)
    return id


@Seat.request("n", [Touch])
def get_touch(self):
    """return touch object

    The ID provided will be initialized to the wl_touch interface
    for this seat.

    This request only takes effect if the seat has the touch
    capability, or has had the touch capability in the past.
    It is a protocol violation to issue this request on a seat that has
    never had the touch capability.

    :returns: :class:`Touch` -- seat touch interface
    """
    id = self._marshal_constructor(2, Touch)
    return id


@Seat.request("5", [])
def release(self):
    """release the seat object

    Using this request a client can tell the server that it is not going to
    use the seat object anymore.
    """
    self._marshal(3)
    self._destroy()


@Seat.event("u", [None])
def capabilities(self, capabilities):
    """seat capabilities changed

    This is emitted whenever a seat gains or loses the pointer,
    keyboard or touch capabilities.  The argument is a capability
    enum containing the complete set of capabilities this seat has.

    When the pointer capability is added, a client may create a
    wl_pointer object using the wl_seat.get_pointer request. This object
    will receive pointer events until the capability is removed in the
    future.

    When the pointer capability is removed, a client should destroy the
    wl_pointer objects associated with the seat where the capability was
    removed, using the wl_pointer.release request. No further pointer
    events will be received on these objects.

    In some compositors, if a seat regains the pointer capability and a
    client has a previously obtained wl_pointer object of version 4 or
    less, that object may start sending pointer events again. This
    behavior is considered a misinterpretation of the intended behavior
    and must not be relied upon by the client. wl_pointer objects of
    version 5 or later must not send events if created before the most
    recent event notifying the client of an added pointer capability.

    The above behavior also applies to wl_keyboard and wl_touch with the
    keyboard and touch capabilities, respectively.

    :param capabilities: capabilities of the seat
    :type capabilities: `uint`
    """
    self._post_event(0, capabilities)


@Seat.event("2s", [None])
def name(self, name):
    """unique identifier for this seat

    In a multiseat configuration this can be used by the client to help
    identify which physical devices the seat represents. Based on
    the seat configuration used by the compositor.

    :param name: seat identifier
    :type name: `string`
    """
    self._post_event(1, name)


Seat._gen_c()
