# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.interface import Interface, InterfaceMeta
from .surface import Surface

import six


@six.add_metaclass(InterfaceMeta)
class Touch(Interface):
    """touchscreen input device

    The wl_touch interface represents a touchscreen
    associated with a seat.

    Touch interactions can consist of one or more contacts.
    For each contact, a series of events is generated, starting
    with a down event, followed by zero or more motion events,
    and ending with an up event. Events relating to the same
    contact point can be identified by the ID of the sequence.
    """
    name = "wl_touch"
    version = 6


@Touch.request("3", [])
def release(self):
    """release the touch object"""
    self._marshal(0)
    self._destroy()


@Touch.event("uuoiff", [None, None, Surface, None, None, None])
def down(self, serial, time, surface, id, x, y):
    """touch down event and beginning of a touch sequence

    A new touch point has appeared on the surface. This touch point is
    assigned a unique ID. Future events from this touch point reference
    this ID. The ID ceases to be valid after a touch up event and may be
    reused in the future.

    :param serial: serial number of the touch down event
    :type serial: `uint`
    :param time: timestamp with millisecond granularity
    :type time: `uint`
    :param surface: surface touched
    :type surface: :class:`Surface`
    :param id: the unique ID of this touch point
    :type id: `int`
    :param x: surface-local x coordinate
    :type x: `fixed`
    :param y: surface-local y coordinate
    :type y: `fixed`
    """
    self._post_event(0, serial, time, surface, id, x, y)


@Touch.event("uui", [None, None, None])
def up(self, serial, time, id):
    """end of a touch event sequence

    The touch point has disappeared. No further events will be sent for
    this touch point and the touch point's ID is released and may be
    reused in a future touch down event.

    :param serial: serial number of the touch up event
    :type serial: `uint`
    :param time: timestamp with millisecond granularity
    :type time: `uint`
    :param id: the unique ID of this touch point
    :type id: `int`
    """
    self._post_event(1, serial, time, id)


@Touch.event("uiff", [None, None, None, None])
def motion(self, time, id, x, y):
    """update of touch point coordinates

    A touch point has changed coordinates.

    :param time: timestamp with millisecond granularity
    :type time: `uint`
    :param id: the unique ID of this touch point
    :type id: `int`
    :param x: surface-local x coordinate
    :type x: `fixed`
    :param y: surface-local y coordinate
    :type y: `fixed`
    """
    self._post_event(2, time, id, x, y)


@Touch.event("", [])
def frame(self):
    """end of touch frame event

    Indicates the end of a set of events that logically belong together.
    A client is expected to accumulate the data in all events within the
    frame before proceeding.

    A wl_touch.frame terminates at least one event but otherwise no
    guarantee is provided about the set of events within a frame. A client
    must assume that any state not updated in a frame is unchanged from the
    previously known state.
    """
    self._post_event(3)


@Touch.event("", [])
def cancel(self):
    """touch session cancelled

    Sent if the compositor decides the touch stream is a global
    gesture. No further events are sent to the clients from that
    particular gesture. Touch cancellation applies to all touch points
    currently active on this client's surface. The client is
    responsible for finalizing the touch points, future touch points on
    this surface may reuse the touch point ID.
    """
    self._post_event(4)


@Touch.event("6iff", [None, None, None])
def shape(self, id, major, minor):
    """update shape of touch point

    Sent when a touchpoint has changed its shape.

    This event does not occur on its own. It is sent before a
    wl_touch.frame event and carries the new shape information for
    any previously reported, or new touch points of that frame.

    Other events describing the touch point such as wl_touch.down,
    wl_touch.motion or wl_touch.orientation may be sent within the
    same wl_touch.frame. A client should treat these events as a single
    logical touch point update. The order of wl_touch.shape,
    wl_touch.orientation and wl_touch.motion is not guaranteed.
    A wl_touch.down event is guaranteed to occur before the first
    wl_touch.shape event for this touch ID but both events may occur within
    the same wl_touch.frame.

    A touchpoint shape is approximated by an ellipse through the major and
    minor axis length. The major axis length describes the longer diameter
    of the ellipse, while the minor axis length describes the shorter
    diameter. Major and minor are orthogonal and both are specified in
    surface-local coordinates. The center of the ellipse is always at the
    touchpoint location as reported by wl_touch.down or wl_touch.move.

    This event is only sent by the compositor if the touch device supports
    shape reports. The client has to make reasonable assumptions about the
    shape if it did not receive this event.

    :param id: the unique ID of this touch point
    :type id: `int`
    :param major: length of the major axis in surface-local coordinates
    :type major: `fixed`
    :param minor: length of the minor axis in surface-local coordinates
    :type minor: `fixed`
    """
    self._post_event(5, id, major, minor)


@Touch.event("6if", [None, None])
def orientation(self, id, orientation):
    """update orientation of touch point

    Sent when a touchpoint has changed its orientation.

    This event does not occur on its own. It is sent before a
    wl_touch.frame event and carries the new shape information for
    any previously reported, or new touch points of that frame.

    Other events describing the touch point such as wl_touch.down,
    wl_touch.motion or wl_touch.shape may be sent within the
    same wl_touch.frame. A client should treat these events as a single
    logical touch point update. The order of wl_touch.shape,
    wl_touch.orientation and wl_touch.motion is not guaranteed.
    A wl_touch.down event is guaranteed to occur before the first
    wl_touch.orientation event for this touch ID but both events may occur
    within the same wl_touch.frame.

    The orientation describes the clockwise angle of a touchpoint's major
    axis to the positive surface y-axis and is normalized to the -180 to
    +180 degree range. The granularity of orientation depends on the touch
    device, some devices only support binary rotation values between 0 and
    90 degrees.

    This event is only sent by the compositor if the touch device supports
    orientation reports.

    :param id: the unique ID of this touch point
    :type id: `int`
    :param orientation: angle between major axis and positive surface y-axis in degrees
    :type orientation: `fixed`
    """
    self._post_event(6, id, orientation)


Touch._gen_c()
